# Generated by the protocol buffer compiler.  DO NOT EDIT!

from google.protobuf import descriptor
from google.protobuf import message
from google.protobuf import reflection
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)



DESCRIPTOR = descriptor.FileDescriptor(
  name='hello.proto',
  package='relay',
  serialized_pb='\n\x0bhello.proto\x12\x05relay\"\x18\n\x05Hello\x12\x0f\n\x07version\x18\x01 \x02(\x05\"V\n\x18HostProcessHelloResponse\x12\x12\n\nsession_id\x18\x01 \x02(\x05\x12\x12\n\nrelay_host\x18\x02 \x02(\t\x12\x12\n\nrelay_port\x18\x03 \x02(\t')




_HELLO = descriptor.Descriptor(
  name='Hello',
  full_name='relay.Hello',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='version', full_name='relay.Hello.version', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=22,
  serialized_end=46,
)


_HOSTPROCESSHELLORESPONSE = descriptor.Descriptor(
  name='HostProcessHelloResponse',
  full_name='relay.HostProcessHelloResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='session_id', full_name='relay.HostProcessHelloResponse.session_id', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='relay_host', full_name='relay.HostProcessHelloResponse.relay_host', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='relay_port', full_name='relay.HostProcessHelloResponse.relay_port', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=48,
  serialized_end=134,
)

DESCRIPTOR.message_types_by_name['Hello'] = _HELLO
DESCRIPTOR.message_types_by_name['HostProcessHelloResponse'] = _HOSTPROCESSHELLORESPONSE

class Hello(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _HELLO
  
  # @@protoc_insertion_point(class_scope:relay.Hello)

class HostProcessHelloResponse(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _HOSTPROCESSHELLORESPONSE
  
  # @@protoc_insertion_point(class_scope:relay.HostProcessHelloResponse)

# @@protoc_insertion_point(module_scope)
