# Generated by the protocol buffer compiler.  DO NOT EDIT!

from google.protobuf import descriptor
from google.protobuf import message
from google.protobuf import reflection
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)



DESCRIPTOR = descriptor.FileDescriptor(
  name='command.proto',
  package='relay',
  serialized_pb='\n\rcommand.proto\x12\x05relay\"=\n\x1e\x43lientConnectionOpenProperties\x12\x1b\n\x13\x61ssigned_to_session\x18\x01 \x02(\x05\"4\n\x1e\x43lientConnectionDropProperties\x12\x12\n\nsession_id\x18\x01 \x02(\x05\"\x85\x02\n\x07\x43ommand\x12\x0f\n\x07version\x18\x01 \x02(\x05\x12\x30\n\x0cmessage_type\x18\x02 \x02(\x0e\x32\x1a.relay.Command.MessageType\x12>\n\x0fopen_properties\x18\x03 \x01(\x0b\x32%.relay.ClientConnectionOpenProperties\x12>\n\x0f\x64rop_properties\x18\x04 \x01(\x0b\x32%.relay.ClientConnectionDropProperties\"7\n\x0bMessageType\x12\x13\n\x0f\x43ONNECTION_OPEN\x10\x00\x12\x13\n\x0f\x43ONNECTION_DROP\x10\x01')



_COMMAND_MESSAGETYPE = descriptor.EnumDescriptor(
  name='MessageType',
  full_name='relay.Command.MessageType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='CONNECTION_OPEN', index=0, number=0,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='CONNECTION_DROP', index=1, number=1,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=348,
  serialized_end=403,
)


_CLIENTCONNECTIONOPENPROPERTIES = descriptor.Descriptor(
  name='ClientConnectionOpenProperties',
  full_name='relay.ClientConnectionOpenProperties',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='assigned_to_session', full_name='relay.ClientConnectionOpenProperties.assigned_to_session', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=24,
  serialized_end=85,
)


_CLIENTCONNECTIONDROPPROPERTIES = descriptor.Descriptor(
  name='ClientConnectionDropProperties',
  full_name='relay.ClientConnectionDropProperties',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='session_id', full_name='relay.ClientConnectionDropProperties.session_id', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=87,
  serialized_end=139,
)


_COMMAND = descriptor.Descriptor(
  name='Command',
  full_name='relay.Command',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='version', full_name='relay.Command.version', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='message_type', full_name='relay.Command.message_type', index=1,
      number=2, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='open_properties', full_name='relay.Command.open_properties', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='drop_properties', full_name='relay.Command.drop_properties', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _COMMAND_MESSAGETYPE,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=142,
  serialized_end=403,
)

_COMMAND.fields_by_name['message_type'].enum_type = _COMMAND_MESSAGETYPE
_COMMAND.fields_by_name['open_properties'].message_type = _CLIENTCONNECTIONOPENPROPERTIES
_COMMAND.fields_by_name['drop_properties'].message_type = _CLIENTCONNECTIONDROPPROPERTIES
_COMMAND_MESSAGETYPE.containing_type = _COMMAND;
DESCRIPTOR.message_types_by_name['ClientConnectionOpenProperties'] = _CLIENTCONNECTIONOPENPROPERTIES
DESCRIPTOR.message_types_by_name['ClientConnectionDropProperties'] = _CLIENTCONNECTIONDROPPROPERTIES
DESCRIPTOR.message_types_by_name['Command'] = _COMMAND

class ClientConnectionOpenProperties(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _CLIENTCONNECTIONOPENPROPERTIES
  
  # @@protoc_insertion_point(class_scope:relay.ClientConnectionOpenProperties)

class ClientConnectionDropProperties(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _CLIENTCONNECTIONDROPPROPERTIES
  
  # @@protoc_insertion_point(class_scope:relay.ClientConnectionDropProperties)

class Command(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _COMMAND
  
  # @@protoc_insertion_point(class_scope:relay.Command)

# @@protoc_insertion_point(module_scope)
